#!/usr/local/bin/php
<?php

require_once('../../includes/config.php');

function usage() {
 echo "php gettext2i18ntext.php <language> <file.po> [base_language] [.po_is_latin1]
 
 where
  language can be \"Traditional Chinese\", \"Slovakian\", ...,
  file.po can be gettext/zn_CN.po, gettext/sk.po, ...,
  base_language can be 2 (default for English), 1 (for Hungarian), ...,
  .po_is_latin1 can be 1 (default 'no') if .po is encoded with ISO-8859-1\n";
 exit(1);
}

if ($_SERVER['argc'] != 3 && $_SERVER['argc'] != 4 && $_SERVER['argc'] != 5)
 usage();
$language=$_SERVER['argv'][1];
$pofile=$_SERVER['argv'][2];
$base_language=$_SERVER['argv'][3];
$po_is_latin1=$_SERVER['argv'][4];
if ($base_language=="")
 $base_language=2; // English
if ($po_is_latin1=="")
 $po_is_latin1='no';

echo "Language is $language, PO file is $pofile, ".
 "base language is $base_language, .po_is_latin1 is $po_is_latin1.";

$f=fopen($pofile,"r");
if ($f=="")
 {
  echo "Unable to open $pofile.\n";
  exit(2);
 }

$link=@pg_connect("dbname=$dbname user=$dbuser password=$dbpasswd port=$dbport");
if ($link=="")
 {
  echo "Unable to open database connection.\n";
  exit(3);
 }

$result=pg_query("select * from _language where langnameen='$language'");
$language_id=pg_fetch_result($result,0,'id');
if ($language_id=="")
 {
  echo "No such language defined yet in the _language table.\n";
  echo "Create it first.\n";
  exit(4);
 }

echo "Language id: $language_id\n";

@pg_query($link, "update _language set available='t' where
 id=$language_id");
echo "Language is activated.";

while (!feof($f))
 {
  $line=fgets($f,8192);
  if (substr($line,0,5)=="msgid")
   {
    $msgid=mb_substr($line, 7, mb_strlen($line)-9);
    $msgstr="";
    $line=fgets($f,8192);
    while (mb_substr($line,0,6)!="msgstr")
     {
      $msgid.=mb_substr($line, 1, mb_strlen($line)-3);
      $line=fgets($f,8192);
     }
    
    while (mb_strlen($line)>1 && !feof($f))
     {
      if (mb_substr($line,0,6)=="msgstr")
       $msgstr.=mb_substr($line, 8, mb_strlen($line)-10);
       else $msgstr.=mb_substr($line, 1, mb_strlen($line)-3);
      $line=fgets($f,8192);
     }
    $msgid=str_replace("\\\"", "\"", $msgid);
    $msgid=str_replace("'", "\\'", $msgid);
    $msgstr=mb_ereg_replace("\\\\\"", "\"", $msgstr);
    $msgstr=mb_ereg_replace("'", "\\'", $msgstr);

    // Unfortunately Hungarian texts are stored in ISO-8859-1,
    // not UTF-8 (for historical reasons). So we have to convert
    // the msgid into ISO-8859-1, even if it is given in UTF-8:
    
    if ($base_language==1 && $po_is_latin1=="no")
     $msgid=iconv("UTF-8", "ISO-8859-1", $msgid);
     // Note that the conversion to Hungarian should be ISO-8859-2,
     // but our conversion script does not write this fact into
     // the .po file. So poEdit (or similar tools) cannot decide
     // the appropriate encoding correctly and chooses ISO-8859-1.

    // $msgstr must be in UTF-8 if .po is latin1:
    if ($po_is_latin1==1)
     $msgstr=iconv("ISO-8859-1", "UTF-8", $msgstr);

    echo "msgid: $msgid\n";
    echo "msgstr: $msgstr\n";
     
    $result=@pg_query($link, "select * from i18ntext where
     _language=$base_language and text='$msgid'");
    $numrows=pg_num_rows($result);
    if ($numrows>1)
     echo "WARNING: msgid is used more than once ($numrows times) as a translation!\n";
    if ($numrows==0)
     echo "WARNING: no such msgid found!\n";
    for ($i=0; $i<$numrows; ++$i)
     {
      $id=@pg_fetch_result($result, $i, 'mainid');
      if ($id!="")
       {
        echo "Hungarian (main) id: $id\n";
        $result2=@pg_query($link, "select * from i18ntext where mainid=$id
         and _language=$language_id");
        $iid=@pg_fetch_result($result2,0,'id');
        if ($iid=="")
         {
          echo "No such entry in $language yet. Inserting...\n";
	  echo "...if msgstr is not empty\n";
	  if ($msgstr!="")
	   {
	    pg_query($link, "insert into i18ntext (mainid, _language, text)
	     values ($id, $language_id, '$msgstr')");
	    echo "Successfully inserted.\n";
	   }
	  else
	   {
	    echo "...no, msgstr is empty, no insertion!\n";
	   }
         } // if ($iid=="")
        else
         {
          echo "Found a $language entry on ID #$iid.\n";
	  if ($msgstr!="")
	   {
	    pg_query($link, "update i18ntext set text='$msgstr'
	     where id=$iid;");
	    echo "Successfully updated.\n";
	   }
	  else
	   {
	    pg_query($link, "delete from i18ntext where id=$iid;");
	    echo "Due to msgstr is empty, this entry has been removed.\n";
	   }
         } // else for if ($iid=="")
       } // if ($id!="")
      else
       {
        echo "WARNING: no id found for message!\n";
       }  // else for if ($id!="")
     } // for ($i...
   } // if (substr...
 } // while

fclose($f);

?>