dnl WMI_ALGEBRA_CHECKMAXIMA
dnl
AC_DEFUN(WMI_ALGEBRA_CHECKMAXIMA,[dnl
maximadir=""
maximaleadinglines=0
AC_ARG_WITH(maximadir,
        AC_HELP_STRING([--with-maximadir=PATH],
                [path to maxima binaries]),
        maximadir=${withval})

if test "x$maximadir" = "x"; then
	AC_CHECK_PROG(MAXIMA,maxima,maxima,no)
	maximadir=`which maxima 2>/dev/null`
else
	AC_CHECK_PROG(MAXIMA,maxima,maxima,no,$maximadir)
	maximadir="${maximadir}/maxima"
fi

if test $MAXIMA = "no"; then
	maximaversion=""
	maximaleadinglines=0
else
	maximaversion=`$maximadir --version | cut -d " " -f2`
	if test "$maximaversion" = ""; then
	    maximaversion=`echo "quit();" | $maximadir | grep ^Maxima | awk '{print @S|@2}'`
	    AC_MSG_WARN([
Maxima version had to be detected via a non-standard way, because
"maxima --version" did not work. Please check if version string is correct.])
	    fi
	AC_MSG_RESULT([detected maxima version: $maximaversion])
	maximaversion=`config/v2n $maximaversion `
        echo "(internally will be used as $maximaversion)"
	maximaleadinglines=`echo "solve(x=5);" | $maximadir | nl -ba | grep "(D1)" | awk '{print (@S|@1)-1}'`
	if test "$maximaleadinglines" = ""; then
	    maximaleadinglines=`echo "solve(x=5);" | $maximadir | nl -ba | grep "(%o1)" | awk '{print (@S|@1)-1}'`
	    fi # For version 5.9.1 and above
	# Over 5.9.2 the maximaleadinglines should be incremented by 1:
	if test $maximaversion -gt 50902; then
	    maximaleadinglines=`expr $maximaleadinglines + 1`
	    fi
	    
	maximagclsupport=`echo "build_info();" | $maximadir | grep "lisp-implementation-version" | grep -c GCL`
	
	if test $maximagclsupport = 0; then
    		AC_MSG_WARN([
Maxima should be compiled with GCL to use it with WebMathematics Interactive.
In fact, WMI should work correctly with other Maxima/Lisp implementations,
however you may experience problems.])
#		maximadir=""
#		maximaversion=""
	fi
fi
AC_SUBST(maximadir)
AC_SUBST(maximaversion)
AC_SUBST(maximaleadinglines)
])

# dnl WMI_ALGEBRA_CHECKOCTAVE
# dnl
# AC_DEFUN(WMI_ALGEBRA_CHECKOCTAVE,[dnl
# octavedir=""
# AC_ARG_WITH(octavedir,
#         AC_HELP_STRING([--with-octavedir=PATH],
#                 [path to octave binaries]),
#         octavedir=${withval})
# 
# if test "x$octavedir" = "x"; then
# 	AC_CHECK_PROG(OCTAVE,octave,octave,no)
# 	octavedir=`which octave 2>/dev/null`
# else
# 	AC_CHECK_PROG(OCTAVE,octave,octave,no,octavedir)
# 	octavedir="${octavedir}/octave"
# fi
# 
# if test $OCTAVE = "no"; then
# 	octavedir=""
# fi
# AC_SUBST(octavedir)
# ])


dnl WMI_ALGEBRA_DETECT
dnl
AC_DEFUN(WMI_ALGEBRA_DETECT,[dnl

#AC_ARG_WITH(maxima,
#        AC_HELP_STRING([--with-maxima],
#                [use only maxima for computer algebra system]),
#        algebrasystem=maxima)
#AC_ARG_WITH(octave,
#        AC_HELP_STRING([--with-octave],
#                [use only octave for computer algebra system]),
#        algebrasystem=octave)

if test "x$algebrasystem" = "x"; then
	WMI_ALGEBRA_CHECKMAXIMA
	# WMI_ALGEBRA_CHECKOCTAVE

	if test mapeledir = "" -a mupaddir = "" && test maximadir = "" && test octavedir = "" ; then
		AC_MSG_ERROR([
At least one computer algebra system is required to install WebMathematics Interactive])
	fi
else
	case $algebrasystem in
	maxima)
		WMI_ALGEBRA_CHECKMAXIMA
                if test $maximadir = ""; then
                        AC_MSG_ERROR([
No maxima computer algebra system found...])
                fi
		mapledir=""
		mupaddir=""
		octavedir=""
 	;;
	octave)
		WMI_ALGEBRA_CHECKOCTAVE
                if test $octavedir = ""; then
                        AC_MSG_ERROR([
No octave computer algebra system found...])
                fi
		maximadir=""
 	;;
	esac
AC_SUBST(maximadir)
fi
])

dnl WMI_ALGEBRA_GETSYSTEMLIST(VARNAME)
dnl
AC_DEFUN(WMI_ALGEBRA_GETSYSTEMLIST,[
$1=""
if test "x$maximadir" != "x"; then $1="$$1,maxima"; fi
if test "x$octavedir" != "x"; then $1="$$1,octave"; fi
if test "x$$1" != "x"; then
	$1=`echo $$1 | $AWK '{printf("%s",substr(@S|@1,2))}'`
fi
])
