<?php
// Plotting related functions.

require_once '../includes/misc.php'; // unlinklog(), execlog()
require_once '../includes/formula.php';

function plotparams($xmin, $xmax, $ymin, $ymax, $grid, $is3d, $ispar, $yangle, $xangle)
{
    return "xmin=$xmin&xmax=$xmax&ymin=$ymin&ymax=$ymax&grid=$grid&is3d=$is3d&ispar=$ispar&yangle=$yangle&xangle=$xangle";
}

// This is for making a plot program from an intuitive formula and parameters
function plotprogram($formula, $xmin, $xmax, $ymin, $ymax, $grid, $is3d, $ispar, $yangle, $xangle)
{

    global $samples, $samples_3d;

    if ($grid == "")
    $grid = "yes";

    // gnuplot does not handle "^", just "**"
    $formula = str_replace("^", "**", $formula); 

// What was it for??
//    $xmin = formconv($xmin, 'gnuplot');
//    $xmax = formconv($xmax, 'gnuplot');
//    $ymin = formconv($ymin, 'gnuplot');
//    $ymax = formconv($ymax, 'gnuplot');

    // Default ranges:
    if ($xmin == "")
	$xmin = -10; 
    if ($xmax == "")
	$xmax = 10; 
  
    if ($ymin == "" && $ymax == "")
	$yranges = "";
    else
    { 
	if ($ymin == "")
	    $ymin = $ymax - 20; 
	if ($ymax == "")
	    $ymax = $ymin + 20;

	$yranges = "$ymin:$ymax";
    }

    if ($yranges == "")
    {

	$ymin = -10;
	$ymax = 10;
	$yranges = "$ymin:$ymax";

    }

    $plot_program = "set nokey\n";
    

    
//    $plot_program .= "set lmargin 10\n";
//    $plot_program .= "set bmargin 5\n";
//    $plot_program .= "set rmargin 0\n";
//    $plot_program .= "set tmargin 0\n";

    if ($grid != "no" && $grid != "off")
	$plot_program .= "set grid\n";


    if(!$is3d)
    {
        $plot_program .= "set samples $samples\n";
	$plot_program .= "set isosamples $samples\n";
    }
    else
    {
	$plot_program .= "set samples $samples_3d\n";
	$plot_program .= "set isosamples $samples_3d\n";
    }

    if ($is3d)
    {
	$plot_program .= "set size ratio -1 1\n";
    }
    else
    {
        $plot_program .= "set size ratio -1 0.7\n";
    }

    if($is3d)
    {
	$plot_program .= "set pm3d\nset palette\n";
	$plot_program .= "set view $yangle,$xangle,1\n"; // fixme
    }
    
    $x0 = "first 0";
    $y0 = "first 0";
    if($xmax < 0)
    {
	$x0 = "graph 1";
    }
    else if($xmin > 0)
    {
	$x0 = "graph 0";
    }

    if($ymax < 0)
    {
	$y0 = "graph 1";
    }
    else if($ymin > 0)
    {
	$y0 = "graph 0";
    }

    $plot_program .= "set style line 1 lt 1 lw 1\n".
            	     "set style line 2 lt 3 lw 1\n".
                     "set style line 3 lt -1 lw 1\n".
		     "set arrow 1 from first $xmin,$y0 to first $xmax,$y0  ls 3\n".
		     "set arrow 2 from $x0,first $ymin to $x0,first $ymax  ls 3\n";
    if($is3d)
    {
        $plot_program .= "splot [$xmin:$xmax] [$yranges] $formula\n";
    }
    else
    {
	if($ispar)
	{
	    $plot_program .= "set parametric\n";
	    $plot_program .= "plot [0:1] [$xmin:$xmax] [$yranges] $formula with dots\n";
	}
	else
	{
            $plot_program .= "plot [$xmin:$xmax] [$yranges] $formula with dots\n";
	}
    }

    return $plot_program;
}

// Generates PNG output from gnuplot program in outputfile
function gnuplot($plotprogram, $outputfile)
{
    global $tempdir, $gnuplot_installation, $convert_installation;

    $basename = substr($outputfile, 0, -4);
    $dir_basename = $tempdir . '/' . $basename;
    $pngfilename = $basename . '-0.png';
    $infilename = $dir_basename . '.in';

    // Generating input for gnuplot:
    $file = fopen($infilename, "w");
    fwrite($file, "set term png crop\n");
    fwrite($file, $plotprogram);
    fclose($file);

    // Calling gnuplot:
//    execlog($gnuplot_installation . ' ' . $infilename . ' > ' . $tempdir . '/' . $pngfilename);
    exec($gnuplot_installation . ' ' . $infilename . ' > ' . $tempdir . '/' . $pngfilename);    
//    execlog($gnuplot_installation . ' ' . $infilename . ' > ' . $tempdir . '/' . $pngfilename);
    
    if (@filesize($tempdir . '/' . $pngfilename) == 0)
    {
	unlinklog($infilename);
        unlinklog($tempdir . '/' . $pngfilename);
	return FALSE;
    }
 
    // Cropping the image:
//    execlog('cd ' . $tempdir.'; ' . $convert_installation . ' -crop 0x0 '.
//	$pngfilename . ' png:' . $outputfile);

//    execlog('cd ' . $tempdir . '; ' . $convert_installation . ' -trim ' .
//    $pngfilename . ' png:' . $outputfile);

    exec('cd ' . $tempdir . '; ' . $convert_installation . ' -trim ' .
    $pngfilename . ' png:' . $outputfile);

    // Cleanup:
//    unlinklog($infilename);
//    unlinklog($tempdir . '/' . $pngfilename);
    
    if (!file_exists($tempdir.'/'.$outputfile))
    {
	return FALSE;
    }
    return TRUE;
}

/*
// Generates PNG output from gnuplot program in outputfile
function gnuplot_texmacs($plotprogram, $outputfile)
{
    global $tempdir, $gnuplot_installation, $convert_installation;
    
    $gnuplot_installation2 = "/usr/lib/texmacs/TeXmacs/bin/tm_gnuplot --texmacs";

    $basename = substr($outputfile, 0, -4);
    $dir_basename = $tempdir . '/' . $basename;
    $pngfilename = $basename . '-0.png';
    $infilename = $dir_basename . '.in';

    // Generating input for gnuplot:
    $file=fopen($infilename,"w");
    fwrite($file,"set term png crop\n");
    fwrite($file,$plotprogram);
    fclose($file);
 
    // Calling gnuplot:
    execlog($gnuplot_installation2 . ' ' . $infilename . ' > ' . $tempdir . '/' . $pngfilename);
 
    // Cropping the image:
//    execlog('cd ' . $tempdir . '; ' . $convert_installation . ' -crop 0x0 '.
//	$pngfilename . ' png:' . $outputfile);
    execlog('cd ' . $tempdir . '; ' . $convert_installation . ' -trim ' .
    $pngfilename . ' png:' . $outputfile);

    // Cleanup:
    unlinklog($tempdir . '/' . $pngfilename);
    unlinklog($infilename);
}
*/

/* Not used

// This function is the general function for plotting functions with gnuplot
function plot_function($function, $xmin, $xmax, $ymin, $ymax, $grid)
{

    global $samples;

    if ($grid == "")
	$grid = "yes";

    $function = str_replace("^", "**", $function); // gnuplot does not handle "^", just "**"
/*
    $xmin = formconv($xmin, 'gnuplot');
    $xmax = formconv($xmax, 'gnuplot');
    $ymin = formconv($ymin, 'gnuplot');
    $ymax = formconv($ymax, 'gnuplot');


// Default ranges:
    if ($xmin == "")
	$xmin = -10; 
    if ($xmax == "")
	$xmax = 10; 
  
    if ($ymin == "" && $ymax == "")
	$yranges = "";
    else
    { 
	if ($ymin == "")
	    $ymin = $ymax - 20; 
	if ($ymax == "")
	    $ymax = $ymin + 20;

	$yranges = "$ymin:$ymax";
    }

    if ($yranges == "")
    {
	$ymin = -10;
	$ymax = 10;
	$yranges = "$ymin:$ymax";
    }

    $plot_program = "set nokey\n";
    if ($grid != "no" && $grid != "off")
	$plot_program .= "set grid\n";
                
 
    $plot_program .= "set samples $samples\n";
    $plot_program .= "set isosamples $samples\n";
    $plot_program .= "set size 0.7\n".
		     "set style line 1 lt 1 lw 1\n".
            	     "set style line 2 lt 3 lw 1\n".
		     "set style line 3 lt -1 lw 1\n".
		     "set arrow 1 from $xmin,0 to $xmax,0  ls 3\n".
		     "set arrow 2 from 0,$ymin to 0,$ymax  ls 3\n";
 
    $plot_program .= "plot [$xmin:$xmax] [$yranges] $function with dots\n";
    return plot($plot_program, "xmin=$xmin&xmax=$xmax&ymin=$ymin&ymax=$ymax");
}
*/

// function ajaxRefreshPlot(whichid,formula,)
// function ajaxLoadPlot(whichid,formula,xmin,xmax,ymin,ymax,grid)

// This function makes an imagemap in order to zoom gnuplot images.
function imagemap(	$whichid, $formula,
			$xmin, $xmax, $ymin, $ymax, $grid,
			$xsize, $ysize, $ispar)
{
    $answer = "<MAP NAME=\"plotmap_" . $whichid . "\">\n";
    $mag = 1.3;
    $res = 7;
    $edge = 20;
    $xstep_click = floor(($xsize - 2 * $edge) / $res);
    $ystep_click = floor(($ysize - 2 * $edge) / $res);
    $xinterval = $xmax - $xmin;
    $yinterval = $ymax - $ymin;
    $xstep = $xinterval / $res;
    $ystep = $yinterval / $res;
    for($i = 0; $i < $res; ++$i)
    {
        for($j = 0; $j < $res; ++$j)
        {
            $xcenter = ($i + 0.5) * $xstep + $xmin;
            $ycenter = $ymax - ($j + 0.5) * $ystep;
//	$xmin_cur = $xcenter - $xinterval / 2 / $mag;
//	$ymin_cur = $ycenter - $yinterval / 2 / $mag;
//	$xmax_cur = $xcenter + $xinterval / 2 / $mag;
//	$ymax_cur = $ycenter + $yinterval / 2 / $mag;
	    $xmin_cur = $xcenter - ($xcenter - $xmin) / $mag;
            $ymin_cur = $ycenter - ($ycenter - $ymin) / $mag;
            $xmax_cur = $xmin_cur + $xinterval / $mag;
            $ymax_cur = $ymin_cur + $yinterval / $mag;

            $answer .= "<AREA SHAPE=\"rect\" style=\"cursor:pointer;\" COORDS=\"";
	    $answer .= ($i * $xstep_click + $edge) . ",";
            $answer .= ($j * $ystep_click + $edge) . ",";
            $answer .= (($i + 1) * $xstep_click + $edge - 1) . ",";
            $answer .= (($j + 1) * $ystep_click + $edge - 1) . '"';
            $answer .= " onclick=\"ajaxRefreshPlot(" .
                 $whichid . "," .
                 "escapeValue('" . $formula . "')," .
                 $xmin_cur . "," .
                 $xmax_cur . "," .
                 $ymin_cur . "," .
                 $ymax_cur . ",'" .
                 $grid . "','" .
                 $ispar . "')\">\n";
        }
    }
    $answer .= "</MAP>\n";
    return $answer;
}

/* Not used
// This function is the general function for plotting functions with gnuplot
function plot_fpltay($function, $tp, $xmin, $xmax, $ymin, $ymax, $grid)
{
    global $cas, $samples;
 
    if ($cas == 'maple')
	$tp = math_raw("evalf($tp)");
    else if ($cas == 'maxima')
	$tp = math_raw("float($tp)");
 
    if ($grid == "")
	$grid = "yes";

    $function = str_replace("^", "**", $function); // gnuplot does not handle "^", just "**"
    $tp = str_replace("^", "**", $tp); 
    $function = str_replace("pl", "+", $function); // gnuplot does not handle "pl", just "+"
    $function = str_replace("ln", "log", $function); // gnuplot does not handle "ln", just "log"
    $function = str_replace("arctan", "atan", $function); // gnuplot does not handle "arctan", just "atan"

    $xmin = formconv($xmin, 'gnuplot');
    $xmax = formconv($xmax, 'gnuplot'); 

    $ymin = formconv($ymin, 'gnuplot');
    $ymax = formconv($ymax, 'gnuplot'); 
 
    $plot_program = "set nokey\n";
    if ($grid != "no")
    $plot_program .= "set grid\n";
                 
    $plot_program .= "set noborder\n".
		     "set noxtics\n".
		     "set noytics\n".
		     "set style line 1 lt 1 lw 1\n".
		     "set style line 2 lt 3 lw 1\n".
		     "set style line 3 lt -1 lw 1\n".
		     "set arrow 1 from $xmin,0 to $xmax,0  ls 3\n".
		     "set arrow 2 from 0,$ymin to 0,$ymax  ls 3\n";

// Default ranges:
    if ($xmin == "")
	$xmin = -10; 
    if ($xmax == "")
	$xmax = 10; 
    if ($ymin == "" && $ymax == "")
	$yranges = "";
    else
    { 
	if ($ymin == "")
	    $ymin = $ymax - 20; 
	if ($ymax == "")
	    $ymax = $ymin + 20;

	$yranges = "$ymin:$ymax";
    }

//	$plot_program .= "set samples $samples\n";
    $plot_program .= "set size 0.4\n";
    $plot_program .= "plot [$xmin:$xmax] [$yranges] $function ls 1, $tp ls 2\n";
    return plot($plot_program);
}
*/

/* Not used

// This function is the general function for
// plotting 3Dfunctions with gnuplot
function plot3d_function($function, $xmin, $xmax, $ymin, $ymax, $sampl, $grid, $ha, $va, $cbase, $csurf)
{
    if ($grid == "")
	$grid = "yes";

    $function = str_replace("^", "**", $function); // gnuplot does not handle "^", just "**"
    $plot_program = "set nokey\n";

    if ($grid != "no")
        $plot_program .= "set grid\n";
  
  
    if ($sampl == "")
	$sampl = 40; 
    if ($ha == "")
	$ha = 60; 
    if ($va == "")
	$va = 30; 


// Default ranges:
    if ($xmin == "")
	$xmin = -10;

    if ($xmax == "")
	$xmax = 10; 

    if ($ymin == "" && $ymax == "")
	$yranges = "";
    else
    { 
	if ($ymin == "")
	    $ymin = $ymax - 20; 
	if ($ymax == "")
	    $ymax = $ymin + 20;

	$yranges = "$ymin:$ymax";

    }

    if ($cbase == "on" && $csurf == "on")
	$plot_program .= "set contour both\n";
    else
    {
	if ($cbase == "on")
	    $plot_program .= "set contour base\n";
	if ($csurf == "on")
	    $plot_program .= "set contour surface\n";
    }

    $plot_program .= "set samples $sampl\n";
    $plot_program .= "set isosamples $sampl\n";
//	$plot_program .= "set hidden3d\n";
    $plot_program .= "set pm3d; set palette\n";
    $plot_program .= "set view $ha,$va,1\n";
//	$plot_program .= "set surface\n";
    $plot_program .= "splot [$xmin:$xmax] [$yranges] $function \n";
    return plot($plot_program);
}

*/

// Complex plotting, cplot.c version 2.0
function cplot($function, $xmin, $xmax, $ymin, $ymax, $grid)
{
    global $tempdir, $convert_installation;

    // Default ranges:
    if($xmin == "")
	$xmin = -10;

    if($xmax == "")
	$xmax = 10;

    if($ymin == "")
	$ymin = -10;

    if($ymax == "")
	$ymax = 10;

    $complex_formula = formconvc($function, 'intuitive', 'ccomplex', '');
//var_dump($complex_formula);
    $complex_formula = "z*z*z";

    // TODO: integrate with webroot/main.php's plotter()
    $plotfile = isincache('complex-png', "", $complex_formula);
    if ($plotfile)
    {
	return $plotfile;
    }

    $plotfile = puttocache('complex-png', "", $complex_formula);
    if(!$plotfile) // failed to put to cache
    {
	$basename = "complex-tmp";
    }
    else
    {
	$basename = substr($plotfile, 0, strpos($plotfile, ".png"));
    }

    $dir_basename = $tempdir . '/' . $basename;
    $pnmfilename = $basename . '.pnm';
    $pngfilename = $basename . '.png';
    $c1filename = $dir_basename . '-1.c'; // cplot.c
    $c2filename = $dir_basename . '-2.c'; // formconv-generated .c
    $hfilename = $dir_basename . '.h'; // formconv-generated .c

    $f = @fopen($c2filename, "w");
    if($f)
    {
	@fwrite($f, $complex_formula . "\n");
    }
    @fclose($f);

    $f = @fopen($hfilename, "w");
    if($f)
    {
	@fwrite($f, "#define XMIN (double)$xmin\n");
	@fwrite($f, "#define XMAX (double)$xmax\n");
	@fwrite($f, "#define YMIN (double)$ymin\n");
	@fwrite($f, "#define YMAX (double)$ymax\n");
	@fwrite($f, "#define SIZE 300.0\n");
    }
    @fclose($f);

    exec("cat c/cplot.c | sed s/\"#include \\\"function\"/\"#include \\\"$basename-2\"/ ".
         " | sed s/\"#include \\\"resolution\"/\"#include \\\"$basename\"/ > $c1filename");
    exec("cd $tempdir && gcc -o $basename $c1filename -lm");
    exec("cd $tempdir && ./$basename > $pnmfilename");
    exec("cd $tempdir && $convert_installation $pnmfilename png:$pngfilename");

/* TODO: error handling
    if (error)
    {
        clearfromcache('complex-png', "", $complex_formula);
        return "Error:complex-plot";
    }
*/

// Cleaning up:

    unlinklog($c1filename);
    unlinklog($c2filename);
    unlinklog($hfilename);
    unlinklog($tempdir . '/' . $basename);
    unlinklog($tempdir . '/' . $pnmfilename);

    return $pngfilename;
}

?>
