<?php
// Various sql functions

require_once '../includes/log.php';

$sql_cached = FALSE;

function connect()
{
    global $link, $includefrom, $dbname, $dbuser, $dbpasswd, $dbport, $dbhost;
/* FIXME
    if($link && is_resource($link))
    {
      return FALSE;
    }
*/
    write_log("Connecting to database $dbname as user $dbuser", LOG_SQL);
    $link = @pg_connect("dbname=$dbname user=$dbuser password=$dbpasswd port=$dbport host=$dbhost");
    if (!$link)
    {
        write_log("PHP could not connect to the database!", LOG_ERROR);
	if($includefrom != 'install.php')
	{
    	    $fatal_error = "DB_ERROR";
    	    include 'error.php';
	    die();
	}
    }
    
    return TRUE;
}

function hasResult(&$result_resource)
{
    if (!$result_resource)
    {
	write_log("SQL error: " . @pg_last_error(), LOG_ERROR);
	print "[phperror]";
	print @pg_last_error();
	print "[/phperror]";
	return FALSE;
    }
    else if (@pg_num_rows($result_resource) == 0)
    {
	write_log("No such entries in the database", LOG_ERROR);
	print "[phperror]No such entries in the database!\n[/phperror]";
	return FALSE;
    }
    return TRUE;
}

// functions related to internationalization

function getmessage($handle, $lang)
{

    global $sql_cached, $lids, $messages;

    if(!$sql_cached)
    {
	$handle = pg_escape_string($handle);
	$mres = db_query("select textid from messages where handle='$handle'");
	if (!hasResult($mres))
	    return FALSE;

	$mid = @pg_fetch_result($mres, 0, 0);
    }
    else
    {
    
	$mid = $messages[$handle];
    
    }

    if(!$sql_cached)
    {
	$lang = pg_escape_string($lang);
	$lres = db_query("select id from _language where langnameen='$lang'");
	if (!hasResult($lres))
	    return FALSE;
	$lid = @pg_fetch_result($lres, 0, 0);
    }
    else
    {
    
	$lid = $lids[$lang];
    
    }

    // Picking up text from the i18ntext table. See the FIXME message below as well.

    $resulti18n = db_query("select text from i18ntext where mainid=$mid and _language=$lid");

    // If no translation found for language (and $lid!=1), then we use the English ($lid==2) text instead:
    if (!$resulti18n || pg_num_rows($resulti18n) == 0)
    {
	$resulti18n = db_query("select text from i18ntext where mainid=$mid and _language=2");
    }
    
    // As a final fallback, we use the Hungarian text:
    if (!$resulti18n || pg_num_rows($resulti18n) == 0)
    {
	$resulti18n = db_query("select text from i18ntext where id=$mid");
    }
    
    // FIXME. This logic is a general technique from version 2.1.5 when many additional languages
    // were introduced. Unfortunately, this kind of technique must be done at 5 or 6 more
    // places in calc.php. Volunteers are needed to make this code more readable and elegant.
    //                                                                     --- Zoltan, 2009-07-04

    if (!hasResult($resulti18n))
	return FALSE;

    if ($lid==1)
     return iconv('ISO-8859-2', 'UTF-8', @pg_fetch_result($resulti18n, 0, 0));
    else
     return @pg_fetch_result($resulti18n, 0, 0);
}

function getlangname($langnameen) 
{

    $langnameen = pg_escape_string($langnameen);
    $lres = db_query("select langnameown from _language where langnameen='$langnameen'");
    if (!hasResult($lres))
	return FALSE;
    if ($langnameen=='Hungarian')   
      return iconv('ISO-8859-2', 'UTF-8', @pg_fetch_result($lres, 0, 0));
    else
      return @pg_fetch_result($lres, 0, 0);
}


// SQL query with logging
function db_query($query)
{

    global $link;

    write_log("SQL Query: $query", LOG_SQL);
    $q = @pg_query($query);
    if(!$q)
    {
	write_log("SQL Query Error: $query", LOG_ERROR);
	write_log("The error was: " . @pg_last_error(), LOG_ERROR);
    }
    return $q;

}

// Caching languages

$lids = array();
$messages = array();

connect();
if($includefrom != 'main.php' && $includefrom != 'install.php') // Don't do caching for these files
{
    $q = db_query("SELECT id, langnameen FROM _language");
    while($row = @pg_fetch_array($q))
    {

	$lids[$row[1]] = $row[0];

    }
    $q = db_query("SELECT textid, handle FROM messages");
    while($row = @pg_fetch_array($q))
    {

	$messages[$row[1]] = $row[0];

    }

    if(count($lids) > 0 && count($messages) > 0)
    {

	$sql_cached = TRUE;
    }

}

?>
