<?php
// Main server-side program, outputs ajax

session_start();

require_once '../includes/config.php';
require_once '../includes/session.php';
$includefrom = 'main.php';
require_once '../includes/sql.php';
require_once '../includes/misc.php';
//require_once '../includes/text.php'; // included from other file
include '../includes/formula.php';
include '../includes/calc.php';
include '../includes/cas.php';
include '../includes/plot.php';
include '../includes/cache.php';
include '../includes/adverts.php';

// General function to plot formulas
function plotter($formula, $xmin, $xmax, $ymin, $ymax, $grid, $is3d, $ispar, $yangle, $xangle)
{
    global $wwwtemp;

    $gnuplot_formula = formconv($formula, 'intuitive', 'gnuplot', '-x aa,.. -x bb,$');

    $parameters = plotparams($xmin, $xmax, $ymin, $ymax, $grid, $is3d, $ispar, $yangle, $xangle);
    $plotfile = isincache('ignuplot-png', $parameters, $gnuplot_formula);
    if (!$plotfile)
    {
	$plotfile = puttocache('ignuplot-png', $parameters, $gnuplot_formula);
	$plotprogram = plotprogram($gnuplot_formula, $xmin, $xmax, $ymin, $ymax, $grid, $is3d, $ispar, $yangle, $xangle);
	$isok = gnuplot($plotprogram, $plotfile);
        if ($isok == FALSE)
	{
	    clearfromcache('ignuplot-png', $parameters, $gnuplot_formula);
	    return "Error:gnuplot";
	}
    }
    return $plotfile;
}

// Creates a maxima call out of a formula string value and an operation string
function withop($value, $operation)
{
    if (strpos($value, ",", 0) != NULL && 
	substr_count($operation, "#") > 1)
    {	
	$last = FALSE;
        $call0 = $operation;
        $oppos = 0;
        $fostart = 0;
        $oppos = strpos($call0, "#", $oppos);

        $foend = strpos($value, ",", $fostart);
        while ($oppos != FALSE && $foend != FALSE)
	{
	    $formula = substr($value, $fostart, $foend - $fostart);
            $call0 = substr_replace($call0,
    			    	    $formula,
    				    $oppos,
				    1);
	    if ($foend == strlen($value))
		return $call0;

    	    $fostart = $foend + 1;
	    $oppos = strpos($call0, "#", $oppos);
            $foend = strpos($value, ",", $fostart);
	    if ($foend == FALSE)
	    {
		if ($last != TRUE)
		{
		    $last = TRUE;
		    $foend = strlen($value);
		}
	    }
        }
        return $call0;
    }
    else
    {
	return str_replace('#', $value, $operation);
    }
}

// Produces the output of a maxima call in string format
function calcopCAS($formula, $operation)
{
    if (isFormconvException($formula))
    {
	return "ERROR:formconv"; // FIXME
    }
    $call = withop($formula, $operation);
    $res = maxima_string($call);
    $res = matrixfilter($res);
    return $res;
}

// Produces the output of a maxima call in TeX/PNG format
function calcop($formula, $operation, $density)
{
    global $wwwtemp;

    if (isFormconvException($formula))
    {
	return "ERROR:formconv"; // FIXME
    }

    $call = withop($formula, $operation);

    if ($_SESSION['calclayout'] == "easy") {
	$call .= "$ res: %$ if featurep(res, real) then res else und";
    }

    $texfile = isincache('maximacall-tex', '', $call);
    if (!$texfile)
    {
        $texfile = puttocache('maximacall-tex', '', $call);
	$texdkeplet = maxima($call);
	if ($texdkeplet == FALSE || $texdkeplet == "$$\%$$")
	{
            clearfromcache('maximacall-tex', '', $call);
	    return "%";
	}
	else if ($texdkeplet == "ERROR:timeout" || $texdkeplet == "RETURN:undefined" || $texdkeplet == "RETURN:ind" ||
		 $texdkeplet == "RETURN:false" || $texdkeplet == "RETURN:true" || $texdkeplet == "RETURN:nosolution" ||
		 $texdkeplet == "RETURN:div0" || $texdkeplet == "RETURN:log0")
	{
            clearfromcache('maximacall-tex', '', $call);
	    return $texdkeplet;
	}
	$texdkeplet = matrix_remove_lrangle($texdkeplet);
	$texdkeplet = colortex($texdkeplet, "blue");
	maketex($texdkeplet, $texfile);
    }

    $pngfile = isincache('maximacall-png', $density, $call);
    if (!$pngfile)
    {
        $pngfile = puttocache('maximacall-png', $density, $call);
        $pngok = tex2png($texfile, $pngfile, $density);
        if (strcmp($pngok, "FALSE") == 0)
	{
            clearfromcache('maximacall-png', $density, $call);
	    return "ERROR:bigimage";
	}
    }
    return './' . $wwwtemp . '/' . $pngfile;
}

// Produces the input formula in TeX/PNG format
function calcact($value, $density, $usecursor)
{
    global $wwwtemp;

    $formula = $value;
    if ($usecursor == false)
    {
        $formula = str_replace(" bb ", "", $formula);
    }
    $texfile = isincache('intuitive-tex', '', $formula);
    $texkeplet = "";
    if (!$texfile)
    {
        $texkeplet = formconv($formula, 'intuitive', 'latex', '-x aa,.. -x bb,$');
	if($texkeplet == "ERROR: formconv"){

	  return "./image/fcex.png";

	  }

	$texkeplet = str_replace("\cdot $", "$", $texkeplet);
	$texkeplet = str_replace("\cdot bb", " bb ", $texkeplet);

    }
    if (!isFormconvException($texkeplet))
    {
	if (!$texfile)
	{
	    // FIXME: very first puttocache fails here
            $texfile = puttocache('intuitive-tex', '', $formula);
            maketex(formtex($texkeplet), $texfile);
	}
        $pngfile = isincache('intuitive-png', $density, $formula);
        if (!$pngfile)
	{
            $pngfile = puttocache('intuitive-png', $density, $formula);
            $pngok = tex2png($texfile, $pngfile, $density);
	    if (strcmp($pngok ,"FALSE") == 0)
	    {
	        clearfromcache('intuitive-png', $density, $formula);
		return './image/fcex.png';
	    }
        }
        return './' . $wwwtemp . '/' . $pngfile;
    }
    else
    {
	return './image/fcex.png';
    }
}

// Creates the div of the teachers
function helptext()
{
    print "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\"><tr><td>";
    print "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\"";
    print " width=\"100%\"><tr><td><b>\n";
//    print "Kategoria: ".getCategories($_SESSION['calclayout']).".\n";
    if ($_SESSION['geoip_city'] == NULL)
    {
	print getmessage("ws_gethelp4", $_SESSION['lang']);
    }
    else
    {
        printf (getmessage("ws_helpdiv", $_SESSION['lang']),
				        $_SESSION['geoip_city']);
    }
    print "</b><br>\n";
    print "</td><td width=\"20\" align=\"right\" valign=\"top\" id=\"xin\" name=\"xin\">\n";
    print "</td></tr></table>\n";
    print "</td></tr>";
    $teachers = getFirstTeachers($_SESSION['calclayout']);
    print "<tr><td>";
    print "<table border=\"0\" cellpadding=\"2\" cellspacing=\"1\"";
    print " width=\"100%\">";
    
    if ($teachers != NULL)
    {
        $insert = "";
	$geoipcity = pg_escape_string($_SESSION['geoip_city']);
	$geoipcity = iconv("UTF-8", "ISO-8859-2", $geoipcity);
	for (	$i = 0; 
		$i < ((3 < count($teachers)) ? 3 : count($teachers));
		$i++)
	{
//	$timezone = date("Z");
//	$timezonehourminute = date("H-i", $timezone);
//	$now = date("Y-m-d H-i-s " . $timezonehourminute);
	    $now = date("Y-m-d H:i:s");
	    $insert .=  "INSERT INTO showed_ad ".
			"(felhasznalo, category, showed, visitor_ip, visitor_geoip_city, clicked) VALUES".
			"(" . $teachers[$i][6] . ", " . $teachers[$i][5] . ", '" . $now . "', '".
			$_SERVER['REMOTE_ADDR'] . "', '" . $geoipcity . "', false); ";
	 
            print "<tr><td>\n";
    	    print "<a onclick=\"ajaxPopupTeacher('" . $teachers[$i][0] . "', '" . $_SERVER['REMOTE_ADDR'] . "', '" . $now . "')\"";
	    print " style=\"cursor:pointer;\">";
	    print "<img src=\"" . $teachers[$i][1] . "\" alt=\"\">";
	    print "</a>";
	    print "</td><td>\n";
	    print "<a onclick=\"ajaxPopupTeacher('" . $teachers[$i][0] . "', '" . $_SERVER['REMOTE_ADDR'] . "', '" . $now . "')\"";
	    print " style=\"cursor:pointer;font-size:smaller;\">";
	    print $teachers[$i][0];
	    print ", ";
	    print $teachers[$i][4];
	    print "</a><br>\n";
	    print "</td></tr>\n";
	}
	db_query($insert);
    }

    print "</table>";
    print "</td></tr><tr><td>";	
    print "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\">";
    print "<tr><td align=\"left\">\n";
    print "<a onclick=\"popupAllTeachers()\" style=\"cursor:pointer;font-size:smaller;text-decoration:none;color:#808080;\">";
    print getmessage("ws_morehelp", $_SESSION['lang']);
    print "</a><br>\n";
    print "</td><td align=\"right\">\n";
    print "<a href=\"http://matek.hu/hirdetes\" target=\"_blank\"";
    print " style=\"cursor:pointer;font-size:smaller;text-decoration:none;color:#808080;\">";
    print getmessage("ws_whatisit", $_SESSION['lang']);
    print "</a><br>\n";
    print "</td></tr></table>\n";
    print "</td></tr></table>\n";
}

//header('Content-type: text/html; charset=utf-8');

$params = "";

if(count($_POST) > 0)
{
    $params .= "POST: ";
    foreach ($_POST as $key => $value)
    {

        $params .= "$key => $value, ";

    }
}

if(count($_GET) > 0)
{
    $params .= "GET: ";
    foreach ($_GET as $key => $value)
    {

        $params .= "$key => $value, ";

    }
}

write_log("main.php called with parameters $params", LOG_WMIDEBUG);


foreach ($_POST as $key => $value)
{
    if ($key == "calc")
    {
	$value2 = str_replace(" bb ", "", $value);
	$formulavalue = formconvc($value2, 'intuitive', 'maxima', '-x aa,.. -x bb,$');
	$imagefile = calcop($formulavalue, $_POST['operation'], $_POST['density']);
	write_log("\$formulavalue = $formulavalue, \$imagefile = $imagefile", LOG_WMIDEBUG);
	if ($imagefile == "%")
	{
	    print "<center>";
	    print getmessage("ws_caserror", $_SESSION['lang']);
	    print "</center>\n";
	    print "<div id=\"codiv\"><div id=\"co\"></div></div>";
	    print "<div id=\"codivt\" style=\"display:none;\"></div>";
	    print "<div id=\"gethelp\" style=\"display:none;\"></div>";
        }
        else if ($imagefile == "ERROR:timeout")
        {
	    $bst = buttonsolvetext($_POST['opbutton'], $_SESSION['lang']);
	    $cont = "<center>";
	    $cont .= getmessage("ws_timeout", $_SESSION['lang']);
	    $cont .= "</center>";
	    $cont .= "<div id=\"co\"></div>";
	    print str_replace('[co]', $cont, $bst);
	    print "<div id=\"codivt\" style=\"display:none;\"></div>";
	}
	else if ($imagefile == "ERROR:bigimage")
	{
	    print "<center>";
    	    print getmessage("ws_bigimage", $_SESSION['lang']);
	    print "</center>\n";
	    print "<div id=\"codiv\"><div id=\"co\"></div></div>";
	    print "<div id=\"codivt\" style=\"display:none;\"></div>";
	    print "<div id=\"gethelp\" style=\"display:none;\"></div>";
	}
	else if ($imagefile == "ERROR:formconv")
	{
    	    print "<center>";
	    print getmessage("ws_badinput", $_SESSION['lang']);
	    print "</center>";
	    print "<div id=\"codiv\"><div id=\"co\"></div></div>";
	    print "<div id=\"codivt\" style=\"display:none;\"></div>";
	    print "<div id=\"gethelp\" style=\"display:none;\"></div>";
	}
	else if ($formulavalue == "ERROR:formconv_error")
	{
    	    print "<center>";
	    print getmessage("mess_formconv_error", $_SESSION['lang']);
	    print "</center>";
	    print "<div id=\"codiv\"><div id=\"co\"></div></div>";
	    print "<div id=\"codivt\" style=\"display:none;\"></div>";
	    print "<div id=\"gethelp\" style=\"display:none;\"></div>";
	}
	else
	{
	    if ($imagefile == "RETURN:undefined")
	    {
		$bst = buttonsolvetext($_POST['opbutton'], $_SESSION['lang']);
		$cont = "<center>";
		$cont .= getmessage("mess_undefined", $_SESSION['lang']);
	        $cont .= "</center>";
		$cont .= "<div id=\"co\"></div>";
		print str_replace('[co]', $cont, $bst);
		print "<div id=\"codivt\" style=\"display:none;\"></div>";
	    }
	    else if ($imagefile == "RETURN:ind")
	    {
		$bst = buttonsolvetext($_POST['opbutton'], $_SESSION['lang']);
		$cont = "<center>";
		$cont .= getmessage("mess_ind", $_SESSION['lang']);
	        $cont .= "</center>";
		$cont .= "<div id=\"co\"></div>";
		print str_replace('[co]', $cont, $bst);
		print "<div id=\"codivt\" style=\"display:none;\"></div>";
	    }
	    else if ($imagefile == "RETURN:false")
	    {
		$bst = buttonsolvetext($_POST['opbutton'], $_SESSION['lang']);
		$cont = "<center>";
		$cont .= getmessage("mess_false", $_SESSION['lang']);
	        $cont .= "</center>";
		$cont .= "<div id=\"co\"></div>";
		print str_replace('[co]', $cont, $bst);
		print "<div id=\"codivt\" style=\"display:none;\"></div>";
	    }
	    else if ($imagefile == "RETURN:true")
	    {
		$bst = buttonsolvetext($_POST['opbutton'], $_SESSION['lang']);
		$cont = "<center>";
		$cont .= getmessage("mess_true", $_SESSION['lang']);
	        $cont .= "</center>";
		$cont .= "<div id=\"co\"></div>";
		print str_replace('[co]', $cont, $bst);
		print "<div id=\"codivt\" style=\"display:none;\"></div>";
	    }
	    else if ($imagefile == "RETURN:nosolution")
	    {
		$bst = buttonsolvetext($_POST['opbutton'], $_SESSION['lang']);
		$cont = "<center>";
		$cont .= getmessage("mess_nosolution", $_SESSION['lang']);
	        $cont .= "</center>";
		$cont .= "<div id=\"co\"></div>";
		print str_replace('[co]', $cont, $bst);
		print "<div id=\"codivt\" style=\"display:none;\"></div>";
	    }
	    else if ($imagefile == "RETURN:div0")
	    {
		$bst = buttonsolvetext($_POST['opbutton'], $_SESSION['lang']);
		$cont = "<center>";
		$cont .= getmessage("mess_div0", $_SESSION['lang']);
	        $cont .= "</center>";
		$cont .= "<div id=\"co\"></div>";
		print str_replace('[co]', $cont, $bst);
		print "<div id=\"codivt\" style=\"display:none;\"></div>";
	    }
	    else if ($imagefile == "RETURN:log0")
	    {
		$bst = buttonsolvetext($_POST['opbutton'], $_SESSION['lang']);
		$cont = "<center>";
		$cont .= getmessage("mess_log0", $_SESSION['lang']);
	        $cont .= "</center>";
		$cont .= "<div id=\"co\"></div>";
		print str_replace('[co]', $cont, $bst);
		print "<div id=\"codivt\" style=\"display:none;\"></div>";
	    }
	    else
	    {
	        $casres = calcopCAS($formulavalue, $_POST['operation']);
		$title = getmessage("ws_formula", $_SESSION['lang']);
		returnsolvetext($_POST['opbutton'], $imagefile, $_SESSION['lang'], $title);
	        print "<div id=\"codivt\" class=\"codivt\" style=\"display:none;\">";
		print $casres . "#";
		print "</div>\n";
	    }
	    if ($adverts_on == TRUE) {
	        print "<div style=\"text-align:right;\"><div id=\"gethelp\" class=\"gethelp\">\n";

		if ($_SESSION['geoip_city'] == NULL)
	        {
		    print getmessage("ws_gethelp4", $_SESSION['lang']);	
	        }
		else
		{
		    $gh = rand(1, 3);
		    printf (getmessage("ws_gethelp" . $gh, $_SESSION['lang']),
			$_SESSION['geoip_city']);
	        }
	        print "<br></div></div>\n";
	    } else {
	    	print "<div id=\"gethelp\"></div>";
	    }
    	}
    }
    else if ($key == "plot")
    {
	$ispar = FALSE;
	if ($_POST['ispar'] == "true")
	{
	    $ispar = TRUE;
	}
	$formula = str_replace(" bb ", "", $value);
	$imagefile = plotter($formula, $_POST['xmin'], $_POST['xmax'], $_POST['ymin'], $_POST['ymax'], $_POST['grid'], FALSE, $ispar, 0, 0);
	if ($imagefile == "Error:gnuplot")
	{
    	    print "<center>";
	    print getmessage("ws_gnuplot_err", $_SESSION['lang']);
	    print "</center>";
	}
	else
	{
	    $isize = @GetImageSize($tempdir . '/' . $imagefile);
	    $title = getmessage("ws_plot", $_SESSION['lang']);
	    print $title . '<br><div id="cogp'.$_POST['fid'] . '" style="overflow:auto;">';
            print '<div id="comap' . $_POST['fid'] . '">';
            print imagemap($_POST['fid'], $_POST['plot'], 
                           $_POST['xmin'], $_POST['xmax'], $_POST['ymin'], $_POST['ymax'], $_POST['grid'], 
				$isize[0], $isize[1], $_POST['ispar']); // FIXME
            print '</div>';
	    print '<table align="center"><tr><td valign="top">';
	    print '<a id="unzoom' . $_POST['fid'] . '" onclick="';
	    $xmax_new = $_POST['xmax'] + ($_POST['xmax'] - $_POST['xmin']) / 2;
	    $xmin_new = $_POST['xmin'] - ($_POST['xmax'] - $_POST['xmin']) / 2;
	    $ymax_new = $_POST['ymax'] + ($_POST['ymax'] - $_POST['ymin']) / 2;
	    $ymin_new = $_POST['ymin'] - ($_POST['ymax'] - $_POST['ymin']) / 2;
	    print 'ajaxRefreshPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['plot'] . "'), " .
		$xmin_new . ',' .
		$xmax_new . ',' .
		$ymin_new . ',' .
		$ymax_new . ",'" .
		$_POST['grid'] . "')";
	    print '" style="cursor:pointer;">';

	    print '<img src="./icon/stock_zoom-out.';
	    print $imgext;
	    print '" class="link" alt="zoom-out" title="';
	    print getmessage("plot_zout", $_SESSION['lang']);
	    print '">';
	    print '</a>';
	    print '</td><td>';
	    print '<img id="copic' . $_POST['fid'] . '" class="copgp" src="' . $wwwtemp . '/' .$imagefile . 
                  '" usemap="#plotmap_' . $_POST['fid'] . '" alt=""></div>';
	    print '</td></tr></table>';
	}
	print "<div id=\"gethelp\"></div>";
    }
    else if ($key == "rplot") // Refresh plot
    {
	$ispar = FALSE;
	if ($_POST['ispar'] == "true") {
	    $ispar = TRUE;
	}
	$formula = str_replace(" bb ", "", $value);
	$imagefile = plotter($formula, $_POST['xmin'], $_POST['xmax'], $_POST['ymin'], $_POST['ymax'], $_POST['grid'], FALSE, $ispar, 0, 0);
	
	if ($imagefile == "Error:gnuplot")
	{
            print '[phperror]';
	}
	else
	{
	    $isize = @GetImageSize($tempdir.'/'.$imagefile);
            print 'var restext1 = "'.$wwwtemp.'/'.$imagefile."\";\n";
            print 'var restext2 = "';
	    $restext2 = '';
	    $restext2 .= imagemap($_POST['fid'], $_POST['rplot'], 
        	$_POST['xmin'], $_POST['xmax'], $_POST['ymin'], $_POST['ymax'],
		$_POST['grid'],
		$isize[0], $isize[1], $_POST['ispar']); // FIXME
	    print urlencode($restext2);
            print "\";\n";
	    print "var restext3 = function(event) { \n";
	    $xmax_new = $_POST['xmax']+($_POST['xmax']-$_POST['xmin'])/2;
	    $xmin_new = $_POST['xmin']-($_POST['xmax']-$_POST['xmin'])/2;
	    $ymax_new = $_POST['ymax']+($_POST['ymax']-$_POST['ymin'])/2;
	    $ymin_new = $_POST['ymin']-($_POST['ymax']-$_POST['ymin'])/2;
	    print 'ajaxRefreshPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['rplot'] . "'), " .
		$xmin_new . ',' .
		$xmax_new . ',' .
		$ymin_new . ',' .
		$ymax_new . ",'" .
		$_POST['grid'] . "')";
	    print " } \n";
	}
    }
    else if ($key == "cplot") // cplot
    {
	$ispar = FALSE;
	if ($_POST['ispar'] == "true")
	{
	    $ispar = TRUE;
	}
	$formula = str_replace(" bb ", "", $value);
	$imagefile = cplot($formula, $_POST['xmin'], $_POST['xmax'], $_POST['ymin'], $_POST['ymax'], $_POST['grid']);
	// todo: error handling
	
	$isize = @GetImageSize($tempdir . '/' . $imagefile);
	$title = getmessage("ws_plot", $_SESSION['lang']);
	print $title . '<br><div id="cogp'.$_POST['fid'] . '" style="overflow:auto;">';
        print '<div id="comap' . $_POST['fid'] . '">';
// zooming...
/*
        print imagemap($_POST['fid'], $_POST['plot'], 
		       $_POST['xmin'], $_POST['xmax'], $_POST['ymin'], $_POST['ymax'], $_POST['grid'], 
		       $isize[0], $isize[1], $_POST['ispar']); // FIXME
        print '</div>';
*/
        print '<table align="center"><tr><td valign="top">';
/*        print '<a id="unzoom' . $_POST['fid'] . '" onclick="';
        $xmax_new = $_POST['xmax'] + ($_POST['xmax'] - $_POST['xmin']) / 2;
        $xmin_new = $_POST['xmin'] - ($_POST['xmax'] - $_POST['xmin']) / 2;
        $ymax_new = $_POST['ymax'] + ($_POST['ymax'] - $_POST['ymin']) / 2;
        $ymin_new = $_POST['ymin'] - ($_POST['ymax'] - $_POST['ymin']) / 2;
        print 'ajaxRefreshPlot(' . $_POST['fid'] . ", escapeValue('" . 
	    $_POST['plot'] . "'), " .
	    $xmin_new . ',' .
	    $xmax_new . ',' .
	    $ymin_new . ',' .
	    $ymax_new . ",'" .
	    $_POST['grid'] . "')";
	print '" style="cursor:pointer;">';

	print '<img src="./icon/stock_zoom-out.';
	print $imgext;
	print '" class="link" alt="zoom-out" title="';
	print getmessage("plot_zout", $_SESSION['lang']);
	print '">';
	print '</a>';
*/
	print '</td><td>';
//	print '<img id="copic' . $_POST['fid'] . '" class="copgp" src="' . $wwwtemp . '/' .$imagefile . 
//              '" usemap="#plotmap_' . $_POST['fid'] . '" alt=""></div>';
        print '<img src="' . $wwwtemp . '/' .$imagefile .'" alt=""></div>';
	print '</td></tr></table>';

	print "<div id=\"gethelp\"></div>";

	// TODO: make ajax of complex plots
    }
    else if ($key == "plot3")
    {
	$ispar = FALSE;
	if ($_POST['ispar'] == "true")
	{
	    $ispar = TRUE;
	}
	$formula = str_replace(" bb ", "", $value);
	$imagefile = plotter($formula, $_POST['xmin'], $_POST['xmax'], $_POST['ymin'], $_POST['ymax'], $_POST['grid'], TRUE, $ispar, $_POST['yangle'], $_POST['xangle']);
	if ($imagefile == "Error:gnuplot")
	{
    	    print "<center>";
	    print getmessage("ws_gnuplot_err", $_SESSION['lang']);
	    print "</center>";
	}
	else
	{
	    $isize = @GetImageSize($tempdir . '/' . $imagefile);
	    $title = getmessage("ws_plot", $_SESSION['lang']);
	    print $title . '<br><div id="cogp'.$_POST['fid'] . '" style="overflow:auto;">';
	    print '<table align="center"><tr><td valign="top">';
	    print '<a id="unzoom' . $_POST['fid'] . '" onclick="';
	    $xmax_new = $_POST['xmax'] + ($_POST['xmax'] - $_POST['xmin'])/4;
	    $xmin_new = $_POST['xmin'] - ($_POST['xmax'] - $_POST['xmin'])/4;
	    $ymax_new = $_POST['ymax'] + ($_POST['ymax'] - $_POST['ymin'])/4;
	    $ymin_new = $_POST['ymin'] - ($_POST['ymax'] - $_POST['ymin'])/4;
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['plot3'] . "'), " .
		$xmin_new . ',' .
		$xmax_new . ',' .
		$ymin_new . ',' .
		$ymax_new . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$_POST['yangle'] . ',' .
		$_POST['xangle'] . ")";
	    print '" style="cursor:pointer;">';
	    print '<img src="./icon/stock_zoom-out.png';
	    //print $imgext;
	    print '" class="link" alt="zoom-out" title="';
	    print getmessage("plot_zout", $_SESSION['lang']);
	    print '">';
	    print '</a>';
	    
	    print '<a id="zoom' . $_POST['fid'] . '" onclick="';
	    $xmax_new = $_POST['xmax'] - ($_POST['xmax'] - $_POST['xmin'])/4;
	    $xmin_new = $_POST['xmin'] + ($_POST['xmax'] - $_POST['xmin'])/4;
	    $ymax_new = $_POST['ymax'] - ($_POST['ymax'] - $_POST['ymin'])/4;
	    $ymin_new = $_POST['ymin'] + ($_POST['ymax'] - $_POST['ymin'])/4;
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['plot3'] . "'), " .
		$xmin_new . ',' .
		$xmax_new . ',' .
		$ymin_new . ',' .
		$ymax_new . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$_POST['yangle'] . ',' .
		$_POST['xangle'] . ")";
	    print '" style="cursor:pointer;">';
	    print '<img src="./icon/stock_zoom-in.png';
	    //print $imgext;
	    print '" class="link" alt="zoom-in" title="';
	    print getmessage("plot_zin", $_SESSION['lang']);
	    print '">';
	    print '</a><br>';
	    
	    print '<a id="up' . $_POST['fid'] . '" onclick="';
	    $yangle_new = $_POST['yangle'] - 10;
	    if ($yangle_new < 0) $yangle_new = 0;
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['plot3'] . "'), " .
		$_POST['xmin'] . ',' .
		$_POST['xmax'] . ',' .
		$_POST['ymin'] . ',' .
		$_POST['ymax'] . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$yangle_new . ',' .
		$_POST['xangle'] . ")";
	    print '" style="cursor:pointer;">';
	    print '<img src="./icon/stock_up.png';
	    //print $imgext;
	    print '" class="link" alt="up" title="';
	    print getmessage("plot_rtop", $_SESSION['lang']);
	    print '">';
	    print '</a>';

	    print '<a id="down' . $_POST['fid'] . '" onclick="';
	    $yangle_new = $_POST['yangle'] + 10;
	    if ($yangle_new > 180) $yangle_new = 180;
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['plot3'] . "'), " .
		$_POST['xmin'] . ',' .
		$_POST['xmax'] . ',' .
		$_POST['ymin'] . ',' .
		$_POST['ymax'] . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$yangle_new . ',' .
		$_POST['xangle'] . ")";
	    print '" style="cursor:pointer;">';
	    print '<img src="./icon/stock_down.png';
	    //print $imgext;
	    print '" class="link" alt="down" title="';
	    print getmessage("plot_rbottom", $_SESSION['lang']);
	    print '">';
	    print '</a><br>';

	    print '<a id="left' . $_POST['fid'] . '" onclick="';
	    $xangle_new = $_POST['xangle'] - 10;
	    if ($xangle_new < 0) $xangle_new = 350;
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['plot3'] . "'), " .
		$_POST['xmin'] . ',' .
		$_POST['xmax'] . ',' .
		$_POST['ymin'] . ',' .
		$_POST['ymax'] . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$_POST['yangle'] . ',' .
		$xangle_new . ")";
	    print '" style="cursor:pointer;">';
	    print '<img src="./icon/stock_left.png';
	    //print $imgext;
	    print '" class="link" alt="left" title="';
	    print getmessage("plot_rleft", $_SESSION['lang']);
	    print '">';
	    print '</a>';

	    print '<a id="right' . $_POST['fid'] . '" onclick="';
	    $xangle_new = $_POST['xangle'] + 10;
	    if ($xangle_new > 360) $xangle_new = 10;
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['plot3'] . "'), " .
		$_POST['xmin'] . ',' .
		$_POST['xmax'] . ',' .
		$_POST['ymin'] . ',' .
		$_POST['ymax'] . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$_POST['yangle'] . ',' .
		$xangle_new . ")";
	    print '" style="cursor:pointer;">';
	    print '<img src="./icon/stock_right.png';
	    //print $imgext;
	    print '" class="link" alt="right" title="';
	    print getmessage("plot_rright", $_SESSION['lang']);
	    print '">';
	    print '</a><br>';
	    
	    print '<a id="north' . $_POST['fid'] . '" onclick="';
	    $xmax_new = $_POST['xmax'];
	    $xmin_new = $_POST['xmin'];
	    $ymax_new = $_POST['ymax'] + ($_POST['ymax'] - $_POST['ymin'])/10;
	    $ymin_new = $_POST['ymin'] + ($_POST['ymax'] - $_POST['ymin'])/10;
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['plot3'] . "'), " .
		$xmin_new . ',' .
		$xmax_new . ',' .
		$ymin_new . ',' .
		$ymax_new . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$_POST['yangle'] . ',' .
		$_POST['xangle'] . ")";
	    print '" style="cursor:pointer;">';
	    print '<img src="./icon/stock_top.png';
	    //print $imgext;
	    print '" class="link" alt="north" title="';
	    print getmessage("plot_oxp", $_SESSION['lang']);
	    print '">';
	    print '</a>';
	    
	    print '<a id="south'. $_POST['fid'] . '" onclick="';
	    $xmax_new = $_POST['xmax'];
	    $xmin_new = $_POST['xmin'];
	    $ymax_new = $_POST['ymax'] - ($_POST['ymax'] - $_POST['ymin'])/10;
	    $ymin_new = $_POST['ymin'] - ($_POST['ymax'] - $_POST['ymin'])/10;
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['plot3'] . "'), " .
		$xmin_new . ',' .
		$xmax_new . ',' .
		$ymin_new . ',' .
		$ymax_new . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$_POST['yangle'] . ',' .
		$_POST['xangle'] . ")";
	    print '" style="cursor:pointer;">';
	    print '<img src="./icon/stock_bottom.png';
	    //print $imgext;
	    print '" class="link" alt="south" title="';
	    print getmessage("plot_oxm", $_SESSION['lang']);
	    print '">';
	    print '</a><br>';

	    print '<a id="west' . $_POST['fid'] . '" onclick="';
	    $xmax_new = $_POST['xmax'] - ($_POST['xmax'] - $_POST['xmin'])/10;
	    $xmin_new = $_POST['xmin'] - ($_POST['xmax'] - $_POST['xmin'])/10;
	    $ymax_new = $_POST['ymax'];
	    $ymin_new = $_POST['ymin'];
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['plot3'] . "'), " .
		$xmin_new . ',' .
		$xmax_new . ',' .
		$ymin_new . ',' .
		$ymax_new . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$_POST['yangle'] . ',' .
		$_POST['xangle'] . ")";
	    print '" style="cursor:pointer;">';
	    print '<img src="./icon/stock_first.png';
	    //print $imgext;
	    print '" class="link" alt="west" title="';
	    print getmessage("plot_oym", $_SESSION['lang']);
	    print '">';
	    print '</a>';

	    print '<a id="east'. $_POST['fid'] . '" onclick="';
	    $xmax_new = $_POST['xmax'] + ($_POST['xmax'] - $_POST['xmin'])/10;
	    $xmin_new = $_POST['xmin'] + ($_POST['xmax'] - $_POST['xmin'])/10;
	    $ymax_new = $_POST['ymax'];
	    $ymin_new = $_POST['ymin'];
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['plot3'] . "'), " .
		$xmin_new . ',' .
		$xmax_new . ',' .
		$ymin_new . ',' .
		$ymax_new . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$_POST['yangle'] . ',' .
		$_POST['xangle'] . ")";
	    print '" style="cursor:pointer;">';
	    print '<img src="./icon/stock_last.png';
	    //print $imgext;
	    print '" class="link" alt="east" title="';
	    print getmessage("plot_oyp", $_SESSION['lang']);
	    print '">';
	    print '</a><br>';

	    print '</td><td>';
	    print '<img id="copic' . $_POST['fid'] . '" class="copgp" src="' . $wwwtemp . '/' .$imagefile . 
                  '"" alt=""></div>';
	    print '</td></tr></table>';
	}
	print "<div id=\"gethelp\"></div>";
    }
    else if ($key == "rplot3")
    {
	$ispar = FALSE;
	if ($_POST['ispar'] == "true") {
	    $ispar = TRUE;
	}
	$formula = str_replace(" bb ", "", $value);
	$imagefile = plotter($formula, $_POST['xmin'], $_POST['xmax'], $_POST['ymin'], $_POST['ymax'], $_POST['grid'], TRUE, $ispar, $_POST['yangle'], $_POST['xangle']);
	
	if ($imagefile == "Error:gnuplot")
	{
            print '[phperror]';
	}
	else
	{
	    $isize = @GetImageSize($tempdir.'/'.$imagefile);
            print 'var restext1 = "'.$wwwtemp.'/'.$imagefile.'";';
	    print 'var restext3 = function(event) { ';
	    $xmax_new = $_POST['xmax']+($_POST['xmax']-$_POST['xmin'])/4;
	    $xmin_new = $_POST['xmin']-($_POST['xmax']-$_POST['xmin'])/4;
	    $ymax_new = $_POST['ymax']+($_POST['ymax']-$_POST['ymin'])/4;
	    $ymin_new = $_POST['ymin']-($_POST['ymax']-$_POST['ymin'])/4;
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['rplot3'] . "'), " .
		$xmin_new . ',' .
		$xmax_new . ',' .
		$ymin_new . ',' .
		$ymax_new . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$_POST['yangle'] . ',' .
		$_POST['xangle'] . ");";
	    print ' };';
	    
	    print 'var restextin = function(event) { ';
	    $xmax_new = $_POST['xmax']-($_POST['xmax']-$_POST['xmin'])/4;
	    $xmin_new = $_POST['xmin']+($_POST['xmax']-$_POST['xmin'])/4;
	    $ymax_new = $_POST['ymax']-($_POST['ymax']-$_POST['ymin'])/4;
	    $ymin_new = $_POST['ymin']+($_POST['ymax']-$_POST['ymin'])/4;
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['rplot3'] . "'), " .
		$xmin_new . ',' .
		$xmax_new . ',' .
		$ymin_new . ',' .
		$ymax_new . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$_POST['yangle'] . ',' .
		$_POST['xangle'] . ");";
	    print ' };';
	    
	    print 'var restextup = function(event) { ';
	    $yangle_new = $_POST['yangle'] - 10;
	    if ($yangle_new < 0) $yangle_new = 0;
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['rplot3'] . "'), " .
		$_POST['xmin'] . ',' .
		$_POST['xmax'] . ',' .
		$_POST['ymin'] . ',' .
		$_POST['ymax'] . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$yangle_new . ',' .
		$_POST['xangle'] . ");";
	    print ' };';

	    print 'var restextnorth = function(event) { ';
	    $xmax_new = $_POST['xmax'];
	    $xmin_new = $_POST['xmin'];
	    $ymax_new = $_POST['ymax']+($_POST['ymax']-$_POST['ymin'])/10;
	    $ymin_new = $_POST['ymin']+($_POST['ymax']-$_POST['ymin'])/10;
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['rplot3'] . "'), " .
		$xmin_new . ',' .
		$xmax_new . ',' .
		$ymin_new . ',' .
		$ymax_new . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$_POST['yangle'] . ',' .
		$_POST['xangle'] . ");";
	    print ' };';

	    print 'var restextdown = function(event) { ';
	    $yangle_new = $_POST['yangle'] + 10;
	    if ($yangle_new > 180) $yangle_new = 180;
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['rplot3'] . "'), " .
		$_POST['xmin'] . ',' .
		$_POST['xmax'] . ',' .
		$_POST['ymin'] . ',' .
		$_POST['ymax'] . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$yangle_new . ',' .
		$_POST['xangle'] . ");";
	    print ' };';

	    print 'var restextsouth = function(event) { ';
	    $xmax_new = $_POST['xmax'];
	    $xmin_new = $_POST['xmin'];
	    $ymax_new = $_POST['ymax']-($_POST['ymax']-$_POST['ymin'])/10;
	    $ymin_new = $_POST['ymin']-($_POST['ymax']-$_POST['ymin'])/10;
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['rplot3'] . "'), " .
		$xmin_new . ',' .
		$xmax_new . ',' .
		$ymin_new . ',' .
		$ymax_new . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$_POST['yangle'] . ',' .
		$_POST['xangle'] . ");";
	    print ' };';

	    print 'var restextleft = function(event) { ';
	    $xangle_new = $_POST['xangle'] - 10;
	    if ($xangle_new < 0) $xangle_new = 350;
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['rplot3'] . "'), " .
		$_POST['xmin'] . ',' .
		$_POST['xmax'] . ',' .
		$_POST['ymin'] . ',' .
		$_POST['ymax'] . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$_POST['yangle'] . ',' .
		$xangle_new . ");";
	    print ' };';

	    print 'var restextright = function(event) { ';
	    $xangle_new = $_POST['xangle'] + 10;
	    if ($xangle_new > 360) $xangle_new = 10;
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['rplot3'] . "'), " .
		$_POST['xmin'] . ',' .
		$_POST['xmax'] . ',' .
		$_POST['ymin'] . ',' .
		$_POST['ymax'] . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$_POST['yangle'] . ',' .
		$xangle_new . ");";
	    print ' };';

	    print 'var restextwest = function(event) { ';
	    $xmax_new = $_POST['xmax']-($_POST['xmax']-$_POST['xmin'])/10;
	    $xmin_new = $_POST['xmin']-($_POST['xmax']-$_POST['xmin'])/10;
	    $ymax_new = $_POST['ymax'];
	    $ymin_new = $_POST['ymin'];
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['rplot3'] . "'), " .
		$xmin_new . ',' .
		$xmax_new . ',' .
		$ymin_new . ',' .
		$ymax_new . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$_POST['yangle'] . ',' .
		$_POST['xangle'] . ");";
	    print ' };';
	    
	    print 'var restexteast = function(event) { ';
	    $xmax_new = $_POST['xmax']+($_POST['xmax']-$_POST['xmin'])/10;
	    $xmin_new = $_POST['xmin']+($_POST['xmax']-$_POST['xmin'])/10;
	    $ymax_new = $_POST['ymax'];
	    $ymin_new = $_POST['ymin'];
	    print 'ajaxRefresh3DPlot(' . $_POST['fid'] . ", escapeValue('" . 
		$_POST['rplot3'] . "'), " .
		$xmin_new . ',' .
		$xmax_new . ',' .
		$ymin_new . ',' .
		$ymax_new . ",'" .
		$_POST['grid'] . "','" .
		$_POST['ispar'] . "'," .
		$_POST['yangle'] . ',' .
		$_POST['xangle'] . ");";
	    print ' };';
	}
    }
    else if ($key == "refresh")
    {
	if (!isset($_POST['cursor']) || $_POST['cursor'] == "false")
	{
	    $out = calcact($value, $_POST['density'], false);
	}
	else
	{
	    $out = calcact($value, $_POST['density'], true);
	}
    write_log("refresh image \$out = $out", LOG_WMIDEBUG);
    print $out;
    }
    else if ($key == "cbgroup")
    {
	buttongroup($value, $_SESSION['lang']);
    }
    else if ($key == "cblayout")
    {
	if ($value != "default")
	{
            setcookie("layout", $value, time() + 604800);
	    $_SESSION['calclayout'] = $value;
	}
	returnlayout($_SESSION['calclayout'], $_SESSION['lang']);
    }
    else if ($key == "getbuttontypes")
    {
	returnbuttontypes($value);
    }
    else if ($key == "newl")
    {
	if ($value != "default")
	{
            setcookie("layout", $value, time() + 604800);
	    $_SESSION['calclayout'] = $value;
	}

	returnintrotext($_SESSION['calclayout'], $_SESSION['lang']);
    }
    else if ($key == "lang")
    {
	// TODO: from database?
	$langret = button2lang($value);
	if (langavailable($langret) == TRUE)
	{
	    $_SESSION['lang'] = $langret;
	    setcookie("language", $_SESSION['lang'], time() + 604800);
	    printf (getmessage("calc_lang", $_SESSION['lang']),
		getlangname($_SESSION['lang']));
	}
	else
	{
	    print getmessage("nolanguage", $_SESSION['lang']);
	}
    }
    else if ($key == "message")
    {
	print getmessage($value, $_SESSION['lang']);
    }
    else if ($key == "help")
    {
	if ($value == "true" && $adverts_on == TRUE)
	{
	    helptext();
	}
	else
	{
	    print "This feature is not yet implemented.<br><br>
	     Sorry, and please reload the page to make this
	     box disappear. (You cannot close it directly,
	     unfortunately.)<br><br>This box will not be shown
	     in the next bugfix release of WMI2.\n";
	}
    }
    else if ($key == "teacher")
    {
	adclicked($value,$_POST['ip'], $_POST['showed']);
	return teacherinfo($value);
    }
    else if ($key == "allteacher")
    {
	print "$value<br>\n";
	one_city_teachers($value);
    }
    else if ($key == "layoutmsg")
    {
	if ($value != "default")
	{
            setcookie("layout", $value, time() + 604800);
	    $_SESSION['calclayout'] = $value;
	}
	print getlayoutmessage($_SESSION['calclayout'], $_SESSION['lang']);
    }
    else if ($key == "lipreload")
    {
	layoutImagePreload($value);
    }
}

?>
